<?php

/**
 * node actions.
 *
 * @package    authpuppy
 * @subpackage node
 * @author     Philippe April <philippe@philippeapril.com>
 * @author     Geneviève Bastien <gbastien@versatic.net>
 * @version    Bzr: $Id$
 */
class nodeActions extends apActions
{
  public function executeIndex(sfWebRequest $request)
  {
    $this->nodes = Doctrine_Query::create()
      ->select('n.*, (select count(*) from connections where node_id = n.id and status = \''.Connection::$TOKEN_VALIDATED.'\') as num_active_connections')
      ->from('Node n')
      ->orderBy('n.name')
      ->execute(array());
   /* $this->nodes = Doctrine::getTable('Node')
      ->createQuery('a')
      ->execute(); */
  }

  public function executeShow(sfWebRequest $request)
  {
    $this->node = Doctrine::getTable('Node')->find(array($request->getParameter('id')));
    $this->forward404Unless($this->node);
  }

  public function executeNew(sfWebRequest $request)
  {
    $this->form = new NodeForm();
  }

  public function executeCreate(sfWebRequest $request)
  {
    $this->forward404Unless($request->isMethod(sfRequest::POST));

    $this->form = new NodeForm();

    $this->processForm($request, $this->form);
    
    // Because we go to the edit and not the list page after creating, we need symfony to update the value of the 'id' field
    $params = $request->getParameter($this->form->getName());
    $params['id'] = $this->form->getObject()->getId();
    // And bind it again
    $this->form->bind($params);

    $this->setTemplate('edit');
  }

  public function executeEdit(sfWebRequest $request)
  {
    $this->forward404Unless($node = Doctrine::getTable('Node')->find(array($request->getParameter('id'))), sprintf('Object node does not exist (%s).', $request->getParameter('id')));
    $this->form = new NodeForm($node);
  }

  public function executeUpdate(sfWebRequest $request)
  {
    $this->forward404Unless($request->isMethod(sfRequest::POST) || $request->isMethod(sfRequest::PUT));
    $this->forward404Unless($node = Doctrine::getTable('Node')->find(array($request->getParameter('id'))), sprintf('Object node does not exist (%s).', $request->getParameter('id')));
    $this->form = new NodeForm($node);
    
    $this->processForm($request, $this->form);

    $this->setTemplate('edit');
  }

  protected function processForm(sfWebRequest $request, sfForm $form)
  {
    $form->bind($request->getParameter($form->getName()), $request->getFiles($form->getName()));
    if ($form->isValid())
    {
      $params = $request->getParameter("submit");
      
      // If the saved button was pressed, the the submit param would be "save".  Otherwise, we notify that an unknown submit button was pressed
      $save = false;
      if (!isset($params['save'])) {
        $event = $this->dispatcher->notifyUntil(new sfEvent($this, 'nodeform.unknown_post_action', array('form' => $form, 'request' => $request)));
        $save = $event->isProcessed();
      } else {
        $save = true;
      }
      
      if ($save) 
        $node = $form->save();
      else
        $this->getUser()->setFlash('notice', "The data was not saved in the database.");
    }
  }
  
 /**
  * Executes login action
  *
  * @param sfRequest $request A request object
  */
  public function executeLogin(sfWebRequest $request)
  {
    /* Get Node */
    $node = Doctrine_Query::create()
      ->from('Node')
      ->where('lower(gw_id) = ?', strtolower($request->getParameter('gw_id')))
      ->limit(1)
      ->fetchOne();
      
    $event = $this->dispatcher->notifyUntil(new sfEvent($this, 'loginpage.request', array('node'=>$node)));

    // If the event is not processed and there is a node or no node was specified
    if ( ($node || (!$node && (is_null($request->getParameter('gw_id'))))) && !$event->isProcessed()) { 
      
      $this->dispatcher->notify(new sfEvent($this, 'authentication.request', array('node' => $node)));

      /* Provide access to useful variables for form */
      $this->selected_authenticator = $request->getParameter('authenticator');
      $this->gw_id = $request->getParameter('gw_id');
      $this->gw_address = $request->getParameter('gw_address');
      $this->gw_port = $request->getParameter('gw_port');

      /* Provide access to authenticators */
      $this->authenticators = apAuthentication::getAuthenticators();
      if (empty($this->authenticators)) {
        $this->authenticators[] = new apDefaultAuthenticator();
      }
      
      // foreach enabled authentication plugin, initiate them
      foreach ($this->authenticators as $authenticator) {
        $authenticator->initialize($request, $this->getIdentity());
      }
        
      /**   If this request is a POST
       *      foreach enabled authentication plugin, process the form 
       */
      if ($request->isMethod('post'))
      {
        foreach ($this->authenticators as $authenticator) {
          $authenticator->process($this, $request, $this->getIdentity());
        }
      }
      
      // If the identity is set, create the connection and call the actionVx.class.php to do what is next
      if ($this->getIdentity()->isIdentified()) {
          $this->dispatcher->notify(new sfEvent($this, 'authentication.success', array('identity' => $this->getIdentity())));
          $authmethod = $this->getIdentity()->getAuthenticatorType();
          
          // Add a privilege as logged in for the current user
          $this->getUser()->addCredential("Logged_" . $authmethod);
          $this->getUser()->addCredential("Logged");
          $this->getUser()->setAttribute('identity', $this->getIdentity());
          
          if ($node) {
             /* Create connection */ 
              $connection = new Connection();
             
              $connection->setNodeRel($node);
              $connection->setAuthType($authmethod);
              $connection->setAuthSubType($this->getIdentity()->getAuthenticatorSubType());
              $connection->setIdentity($this->getIdentity()->getId());
              
              $event = $this->dispatcher->filter(new sfEvent($this, 'connection.pre_saved', array('connection' => $connection, 'identity' => $this->getIdentity())), $connection);
              $connection = $event->getReturnValue();
              $connection->save();
              
              /* Forward user straight back to router */
              $this->redirect('http://' . $request->getParameter('gw_address') . ':' . $request->getParameter('gw_port') . '/wifidog/auth?token=' . $connection->getToken());
          }
          else {
              $this->redirect('home/index');
          }
          
          return sfView::SUCCESS;
      }

      return sfView::SUCCESS;
    } elseif (!$node && (!is_null($request->getParameter('gw_id')))) {
      $this->getUser()->setFlash('notice', "The node you're trying to access does not exist on the server.");
      $event = $this->dispatcher->notifyUntil(new sfEvent($this, 'loginpage.node_not_found', array('request'=>$request)));
      if ($event->isProcessed())
        return sfView::SUCCESS;
      else
        return sfView::ERROR;
    } else {
      return sfView::ERROR;
    }
  }

 /**
  * Executes portal action
  *
  * @param sfRequest $request A request object
  */
  public function executePortal(sfWebRequest $request)
  {
    /* Get Node */
    $this->node = Doctrine_Query::create()
      ->from('Node')
      ->where('lower(gw_id) = ?', strtolower($request->getParameter('gw_id')))
      ->limit(1)
      ->fetchOne();

    $event = $this->dispatcher->notifyUntil(new sfEvent($this, 'portalpage.request', array('node' => $this->node)));
      
    if ($this->node && !$event->isProcessed()) {
      
      //$this->redirect('http://google.ca');
      return sfView::SUCCESS;
    } elseif (!$this->node && (!is_null($request->getParameter('gw_id')))) {
      $event = $this->dispatcher->notifyUntil(new sfEvent($this, 'portalpage.node_not_found', array('request'=>$request)));
      if ($event->isProcessed())
        return sfView::SUCCESS;
      else
        return sfView::ERROR;
    } else {
      return sfView::ERROR;
    }
  }

  public function executeAuth(sfWebRequest $request)
  {
    # Call the right version
    $version = ($request->getParameter("v") ? $request->getParameter("v") : 1);
    $classname = "apActionsV" . $version;
    $this->messages = "";
    call_user_func(array($classname, 'auth'), $this, $request);
  }

  public function executePing(sfWebRequest $request)
  {
    # Call the right version
    $version = ($request->getParameter("v") ? $request->getParameter("v") : 1);
    $classname = "apActionsV" . $version;
    call_user_func(array($classname, 'ping'), $this, $request);
  }
  
  public function executeGwmessage(sfWebRequest $request) {
    $message = $request->getParameter("message");
    $event = $this->dispatcher->filter(new sfEvent($this, 'node.gw_message', array('message' => $message, 'identity' => $this->getUser()->getAttribute('identity'))), array());
    $this->messages = $event->getReturnValue();
    if (empty($this->messages)) {
      if ($message == 'denied') {
        $this->messages[] = "Access denied";
      }
    }
  }
}
