<?php

/**
 * BaseIdentity
 * 
 * This class is the base class for any network identity class.
 * 
 * @property integer|string $id
 * @property boolean $identified default false
 * @property apAuthentication $authenticator a
 * 
 * @method integer    getId()    Returns the current identity's id
 * @method object			getIdentityObject()  
 * @method -					identify($id, $identityObject)
 * @method -					logout()
 * @method boolean		isIdentified()
 *
 * @package    authpuppy
 * @subpackage model
 * @author     Geneviève Bastien <gbastien@versatic.net>
 * @version    bzr: $Id$
 */
abstract class apBaseIdentity
{
  // Value to uniquely identify this identity for a give authentication type
  protected $_id = null;
  
  // Identity object associated with this identity
  // Is this object really needed?  It implements no common interface and is more something that should belong to the authenticator
  protected $_object = null;
  
  // Whether someone is identified or not
  protected $_identified = false;
  
  // The authenticator object this identity was found with
  protected $_authenticator = null;
  
  /**
   * Returns the id of the identity
   * @return integer|string
   */
  public function getId() {
    return $this->_id;
  }
  
  /**
   * Returns the identity object for a given authentication method
   * @return object
   */
  public function getIdentityObject() {
    return $this->_object;
  }
  
	/**
   * Returns the authenticator object 
   * @return object
   */
  public function getAuthenticator() {
    return $this->_authenticator;
  }
  
  public function getAuthenticatorType() {
    if (is_object($this->_authenticator)) {
      return get_class($this->_authenticator); 
    } 
    return '';
  }
  
  public function getAuthenticatorSubType() {
    if (is_object($this->_authenticator)) {
      return $this->_authenticator->getSubType(); 
    } 
    return '';
  }
  
  /**
   * Abstract function to save the identity of the connection
   * @param $id string|integer
   * @param $identityObject object
   * @return none
   */
  abstract function identify($id, $identityObject = null, $authenticator = null) ;
  
  /**
   * Delete the identity
   * @return none
   */
  abstract function logout();
  
  /**
   * Returns whether there is an identity registered
   * @return boolean
   */
  public function isIdentified() {
    return $this->_identified;
  }
  
}