<?php

/**
 * Implements an authpuppy event listener.  It actually uses the same functionnalities as the
 * symfony event listeners but adds static function calls to globally be able to reach it.
 *
 * @package    authpuppy
 * @subpackage model
 * @author     Geneviève Bastien <gbastien@versatic.net>
 * @version    bzr: $Id$
 */

class apEventDispatcher extends sfEventDispatcher {
  
  protected static $instance = null;
  
  protected static function getInstance() {
    if (is_null(self::$instance)) {
      self::$instance = new apEventDispatcher();
    }
    return self::$instance;
  }
  
  /**
   * Static function to connect a listener to an event, shortcut for the event dispatcher's connect function
   * @see sfEventDispatcher: connect()
   */
  public static function sconnect($name, $listener) {
    self::getInstance()->connect($name, $listener);
  }
  
	/**
   * Static function to disconnect a listener from an event, shortcut for the event dispatcher's disconnect function
   * @see sfEventDispatcher: disconnect()
   */
  public static function sdisconnect($name, $listener) {
    self::getInstance()->disconnect($name, $listener);
  }
  
	/**
   * Static function to notify listeners of an event, shortcut for the event dispatcher's notify function
   * @see sfEventDispatcher: notify()
   */
  public static function snotify(sfEvent $event) {
    self::getInstance()->notify($event);
  }
  
	/**
   * Static function to notify listeners of an event, shortcut for the event dispatcher's notifyUntil function
   * @see sfEventDispatcher: notifyUntil()
   */
  public static function snotifyUntil(sfEvent $event) {
    self::getInstance()->notifyUntil($event);
  }
  
	/**
   * Static function to filter a value with an event, shortcut for the event dispatcher's filter function
   * @see sfEventDispatcher: filter()
   */
  public static function sfilter(sfEvent $event, $value) {
    self::getInstance()->filter($event, $value);
  }
  
  
}