<?php

/**
 * apPluginManagerListForm
 * 
 * Form to list the list of plugins and enable / disable them
 * 
 * 
 * @package    authpuppy
 * @subpackage apPluginManager
 * @author     Geneviève Bastien <gbastien@versatic.net>
 * @version    $Id$
 */

class apPluginManagerListForm extends BaseForm
{
  protected $plugins;
  /**
   * @see sfForm
   */
  public function setup()
  {
    $this->plugins = apPlugins::getInstance();
 
    foreach ($this->plugins->getPlugins() as $pluginName => $plugin) {
      $this->widgetSchema[$pluginName] = new sfWidgetFormInputCheckbox(); 
      $this->validatorSchema[$pluginName] = new sfValidatorPass();
      $this->setDefault($pluginName, $plugin->isEnabled());
      
    }

    $this->widgetSchema->setNameFormat('apPluginManagerList[%s]');
  }
  
  public function save() {
    
    // Set the state of every plugin
    $statuschanged = false;
    foreach ($this->plugins->getPlugins() as $pluginName => $plugin) {
      $val = is_null($this->getValue($pluginName))?false:true ;
      $statuschanged |= $plugin->enable($val);

      if ($val == true) {
        // See if we have one migration for it
        if ($migration = $plugin->getMigrationObject()) {
          if (!$migration->hasMigrated()) {
            $migration->migrate();
          }
        }
      }
    }
    
    // If there was a change in the status of a plugin, save the configuration
    if ($statuschanged) {
      $this->plugins->saveConfiguration();
      
      // If there were changes, after saving, we run the publish-assets task to be sure all assets are in the web directory
      // And the clear cache task
      $webexec = new apWebExecutor();
      $webexec->run('sfPluginPublishAssetsTask');
      $webexec->run('sfCacheClearTask');
    }
  }
  
  public function getPlugins() {
    return $this->plugins->getPlugins();
  }
}
